"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Writer = void 0;
const utils_1 = require("@typescript-eslint/utils");
class Writer {
    guard;
    isTsx;
    options;
    sourceCode;
    constructor(scope, guard) {
        this.guard = guard;
        this.isTsx = scope.isTsx;
        this.options = scope.options;
        this.sourceCode = scope.sourceCode;
    }
    getBodySource({ body }) {
        if (this.options.returnStyle !== 'explicit' && this.guard.isBlockStatementWithSingleReturn(body)) {
            const returnValue = body.body[0].argument;
            const source = this.sourceCode.getText(returnValue);
            return returnValue.type === utils_1.AST_NODE_TYPES.ObjectExpression ? `(${source})` : source;
        }
        if (this.guard.hasImplicitReturn(body) && this.options.returnStyle !== 'implicit') {
            return `{ return ${this.sourceCode.getText(body)} }`;
        }
        return this.sourceCode.getText(body);
    }
    getParamsSource(params) {
        return params.map((param) => this.sourceCode.getText(param));
    }
    getFunctionName(node) {
        return node.id && node.id.name ? node.id.name : '';
    }
    getGenericSource(fn) {
        if (!this.guard.hasTypeParameters(fn))
            return '';
        const genericSource = this.sourceCode.getText(fn.typeParameters);
        if (!this.isTsx)
            return genericSource;
        const params = fn.typeParameters.params;
        if (params.length === 1)
            return `<${params[0].name.name},>`;
        return genericSource;
    }
    getReturnType(node) {
        return node.returnType && node.returnType.range && this.sourceCode.getText().substring(...node.returnType.range);
    }
    writeArrowFunction(node) {
        const fn = this.getFunctionDescriptor(node);
        const ASYNC = fn.isAsync ? 'async ' : '';
        const GENERIC = fn.isGeneric ? fn.generic : '';
        const BODY = fn.body;
        const RETURN_TYPE = fn.returnType ? fn.returnType : '';
        const PARAMS = fn.params.join(', ');
        return `${ASYNC}${GENERIC}(${PARAMS})${RETURN_TYPE} => ${BODY}`;
    }
    writeArrowConstant(node) {
        const fn = this.getFunctionDescriptor(node);
        return `const ${fn.name} = ${this.writeArrowFunction(node)}`;
    }
    getFunctionDescriptor(node) {
        return {
            body: this.getBodySource(node),
            isAsync: this.guard.isAsyncFunction(node),
            isGenerator: this.guard.isGeneratorFunction(node),
            isGeneric: this.guard.hasTypeParameters(node),
            name: this.getFunctionName(node),
            generic: this.getGenericSource(node),
            params: this.getParamsSource(node.params),
            returnType: this.getReturnType(node),
        };
    }
}
exports.Writer = Writer;
//# sourceMappingURL=writer.js.map